# maketables.pl by Fergus Duniho
# This script uses the data in list.txt to construct diagrams for 
# Cavalier Chess problems. An example data entry from list.txt 
# looks like this:
#
# 1
# Kd2 cd3 kd4 qe4
# Red to Move
# Queen e4-e1
#
# The first line gives the number of the problem.
#
# The second line gives the positions of the pieces.
# Each string in the second line indicates a piece and its position.
# The first letter identifies the piece. 
# K=KnightKing, Q=Queen, M=Marshall, P=Paladin, N=Nightrider, C=Cavalier
# Uppercase is used for White pieces, lowercase for red.
# The other two letters identify the coordinates of the pieces.
#
# The third line identifies who is to move. 
# It should say "Red to Move" or "White to Move"
#
# The last line gives the solution. In the solution use the full piece name,
# followed by computer notation for the move. A promotion looks like this:
# Cavalier g7-e8 = Paladin
# Put " = <Name>" after the move for a promotion. Use the full piece name.
#
# This scripts contructs diagrams for both the problem and the solution.
# It calculates positions for the solution from the original position
# and the move given as the solution. Each goes into its own HTML file.
# Problems go into mateinone#.html, and solutions go into solution#.html.
#
# The diagrams are all done as tables. GIF images are used for pieces, and
# board.css, which is a style sheet, is used to color the squares. In each
# table, a square's color is identified as the same as a1, a8, h1, or h8. 
# board.css specifies what color these four squares are. By making them all
# different, you can have four color boards. By making two pairs the same,
# you can have regular chess boards. It is currently set up for three color
# boards.

if (open(LISTFILE, "<list.txt")) {
    print "Opened list.txt\n";
    $n = <LISTFILE>;
    chop ($n);
    while ($n <= 55) {
	$p = <LISTFILE>;
	$m = <LISTFILE>;
	$s = <LISTFILE>;
	chop ($s);
	$f = "mateinone" . $n . ".html";
	$next = "mateinone" . ($n+1) . ".html";
	$prev = "mateinone" . ($n-1) . ".html";
	$solution = "solution" . $n . ".html";
	print "Will try to open $f\n";
	if (open(OUTFILE, ">$f")) {
	    print "Opened $f\n";
	    print OUTFILE "<HTML><HEAD><TITLE>Cavalier Chess Mate-in-One Problem \#$n</TITLE>";
	    print OUTFILE "<LINK REL=StyleSheet HREF=\"board.css\" TYPE=\"text/css\" MEDIA=screen></HEAD><BODY>";
	    print OUTFILE "<A HREF=\"../\"><IMG SRC=\"../logo.gif\" BORDER=0></A>";
	    print OUTFILE "<H1 ALIGN=CENTER>Mate-in-One Problem \#$n</H1>";
	    print OUTFILE "<TABLE ALIGN=LEFT CELLPADDING=0 CELLSPACING=0 BORDER=0><TR><TH WIDTH=49 HEIGHT=49>&nbsp;<TD WIDTH=4><TH WIDTH=49>A<TH WIDTH=49>B<TH WIDTH=49>C<TH WIDTH=49>D<TH WIDTH=49>E<TH WIDTH=49>F<TH WIDTH=49>G<TH WIDTH=49>H<TR><TD><TD COLSPAN=10 ID=B><IMG SRC=\"blank.gif\" HEIGHT=4>";
	    for ($rank = 8; $rank > 0; $rank--) {
		print OUTFILE "<TR><TH HEIGHT=49>$rank<TD ID=B><IMG SRC=\"blank.gif\" WIDTH=4>";
		for ($file = "a"; $file lt "i"; $file = chr(ord($file) + 1)) {
		    $square = $file . $rank;
		    $color = (ord($file) & 1 ? "A" : "H") . ($rank & 1 ? "1" : "8");
		    if (($i = index($p, $square)) < 0) {
			print OUTFILE "<TD ID=$color>&nbsp;";
		    }
		    else {
			$i -= 1;
			$piece = substr($p, $i, 1);
		    if ($piece eq "c") { $image = "BCavalier.gif"; }
			elsif ($piece eq "n") {$image = "BNightrider.gif"; }
			elsif ($piece eq "p") {$image = "BPaladin.gif"; }
			elsif ($piece eq "m") {$image = "BMarshall.gif"; }
			elsif ($piece eq "q") {$image = "BQueen.gif"; }
			elsif ($piece eq "k") {$image = "BKnightKing.gif"; }
			elsif ($piece eq "C") {$image = "WCavalier.gif"; }
		    elsif ($piece eq "N") {$image = "WNightrider.gif"; }
			elsif ($piece eq "P") {$image = "WPaladin.gif"; }
			elsif ($piece eq "M") {$image = "WMarshall.gif"; }
			elsif ($piece eq "Q") {$image = "WQueen.gif"; }
		    elsif ($piece eq "K") {$image = "WKnightKing.gif"; }
			else { $image = "BDownPawn.gif"; }
			print OUTFILE "<TD ID=$color><IMG SRC=\"$image\">";
		    }
		}
		print OUTFILE "<TD ID=B><IMG SRC=\"blank.gif\" WIDTH=4>";
	    }
	    print OUTFILE "<TR><TD><TD COLSPAN=10 ID=B><IMG SRC=\"blank.gif\" HEIGHT=4><TR><TD><TH COLSPAN=10>$m</TABLE>";
	    print OUTFILE "<TABLE ALIGN=RIGHT><TR><TD><A HREF=\"$solution\">Solution</A><TR><TD><A HREF=\"$next\">Next Problem</A><TR><TD><A HREF=\"$prev\">Previous Problem</A></TABLE></BODY></HTML>";
	    close (OUTFILE);
	}
	if (open(OUTFILE, ">$solution")) {
	    print "Opened $f\n";
	    $i = index($s, " ") + 1;
	    $old = substr($s, $i, 2);
	    $new = substr($s, $i+3, 2);
	    $i = index($p, $old);
	    $z = $p;
	    substr($p, $i, 2) = $new;
	    if ($z eq $p) {
		print "ERROR";
		exit;
	    }
	    else {
		print "\$z is $z\n\$p is $p\n";
	    }
	    print OUTFILE "<HTML><HEAD><TITLE>Cavalier Chess Mate-in-One Solution \#$n</TITLE>";
	    print OUTFILE "<LINK REL=StyleSheet HREF=\"board.css\" TYPE=\"text/css\" MEDIA=screen></HEAD><BODY>";
	    print OUTFILE "<A HREF=\"../\"><IMG SRC=\"../logo.gif\" BORDER=0></A>";
	    print OUTFILE "<H1 ALIGN=CENTER>Mate-in-One Solution \#$n</H1>";
	    print OUTFILE "<TABLE ALIGN=LEFT CELLPADDING=0 CELLSPACING=0 BORDER=0><TR><TH WIDTH=49 HEIGHT=49>&nbsp;<TD WIDTH=4><TH WIDTH=49>A<TH WIDTH=49>B<TH WIDTH=49>C<TH WIDTH=49>D<TH WIDTH=49>E<TH WIDTH=49>F<TH WIDTH=49>G<TH WIDTH=49>H<TR><TD><TD COLSPAN=10 ID=B><IMG SRC=\"blank.gif\" HEIGHT=4>";
	    for ($rank = 8; $rank > 0; $rank--) {
		print OUTFILE "<TR><TH HEIGHT=49>$rank<TD ID=B><IMG SRC=\"blank.gif\" WIDTH=4>";
		for ($file = "a"; $file lt "i"; $file = chr(ord($file) + 1)) {
		    $square = $file . $rank;
		    $color = (ord($file) & 1 ? "A" : "H") . ($rank & 1 ? "1" : "8");
		    if (($i = index($p, $square)) < 0) {
			print OUTFILE "<TD ID=$color>&nbsp;";
		    }
		    else {
			$i -= 1;
			$piece = substr($p, $i, 1);
			if ($piece eq "c") { 
			    if ($rank == 1) {
				$i = index($s,"=") + 2;
				$image = "B" . substr($s, $i) . ".gif";
			    }
			    else {
				$image = "BCavalier.gif";
			    }
			}
			elsif ($piece eq "n") {$image = "BNightrider.gif"; }
			elsif ($piece eq "p") {$image = "BPaladin.gif"; }
			elsif ($piece eq "m") {$image = "BMarshall.gif"; }
			elsif ($piece eq "q") {$image = "BQueen.gif"; }
			elsif ($piece eq "k") {$image = "BKnightKing.gif"; }
			elsif ($piece eq "C") { 
			    if ($rank == 8) {
				$i = index($s,"=") + 2;
				$image = "W" . substr($s, $i) . ".gif";
			    }
			    else {
				$image = "WCavalier.gif";
			    }
			}
			elsif ($piece eq "N") {$image = "WNightrider.gif"; }
			elsif ($piece eq "P") {$image = "WPaladin.gif"; }
			elsif ($piece eq "M") {$image = "WMarshall.gif"; }
			elsif ($piece eq "Q") {$image = "WQueen.gif"; }
			elsif ($piece eq "K") {$image = "WKnightKing.gif"; }
			else { $image = "BDownPawn.gif"; }
			print OUTFILE "<TD ID=$color><IMG SRC=\"$image\">";
		    }
		}
		print OUTFILE "<TD ID=B><IMG SRC=\"blank.gif\" WIDTH=4>";
	    }
	    print OUTFILE "<TR><TD><TD COLSPAN=10 ID=B><IMG SRC=\"blank.gif\" HEIGHT=4><TR><TD><TH COLSPAN=10>$s</TABLE>";
	    print OUTFILE "<TABLE ALIGN=RIGHT><TR><TD><A HREF=\"$f\">This Problem</A><TR><TD><A HREF=\"$next\">Next Problem</A><TR><TD><A HREF=\"$prev\">Previous Problem</A></TABLE></BODY></HTML>";
	    close (OUTFILE);
	}
	    
	$n = <LISTFILE>;
	chop ($n);
    }
    close (LISTFILE);
}
